<div class='row'>
  <div class='col-lg-12'>
    <ol class='breadcrumb'>
      <li><a href='/packages/<%= @package.prefix %>'><%= @package.prefix %></a></li>
      <li class='active'><%= @package.name %></li>
    </ol>

    <h2>
      <%= @package.name %>
      <%== erb(:package_links) %>
    </h2>

  <% if @package.blacklisted? %>
    <div class='alert alert-warning'>
      This package is currently <a href="/status/blacklist/">blacklisted</a> and will not have any new test runs.
    </div>
  <% end %>

    <table class="table" style='width: auto'>

      <tr>
        <td></td>
        <% Debci.config.suite_list.each do |suite| %>
          <td><b><%= suite %></b></td>
        <% end %>
      </tr>

      <% Debci.config.arch_list.each_with_index do |arch, i| %>
        <tr>
          <td><b><%= arch %></b></td>
          <% @package.status[i].each do |status| %>
            <td>
              <% if status %>
                <%== icon(status.status) %>
                <a href="<%= history_url(status) %>"><%= status.title %></a>
              <% else %>
                <%== icon(:no_test_data) %>
                No test data
              <% end %>
            </td>
          <% end %>
        </tr>
      <% end %>

    </table>

  </div>
</div>
