<div class='row'>
  <div class='col-md-12'>
    <h1 class='page-header'>Status <small>/ Slow running tests</small></h1>
    <%== @status_nav %>

    <% if @slow.empty? %>
      <em>No items</em>
    <% else %>
      <table class='table table-striped'>
        <tr>
          <th>Package</th>
          <th>Version</th>
          <th>Suite</th>
          <th>Architecture</th>
          <th>Status</th>
          <th>Test time</th>
        </tr>

        <% @slow.each do |status| %>
          <% link = history_url(status) %>
          <tr>
            <td>
              <a href="<%= link %>"><%= status.package.name %></a>
            </td>

            <td>
              <a href="<%= link %>"><%= status.version %></a>
            </td>

            <td>
              <a href="<%= link %>"><%= status.suite %></a>
            </td>

            <td>
              <a href="<%= link %>"><%= status.arch %></a>
            </td>

            <td>
              <%== icon(status.status)%> <%= status.title %>
            </td>

            <td>
              <%= status.duration_human %>
            </td>
          </tr>
        <% end %>
      </table>
    <% end %>

  </div> <!-- class='col-md-12' -->
</div> <!-- class='row' -->
