<div class="row">

  <div class="col-md-6">

    <h3>Packages</h3>
    <ul class="list-inline">
      <% @package_prefixes.each do |prefix| %>
        <li><h5><a href="/packages/<%= prefix %>"><%= prefix %></a></h5></li>
      <% end %>
      <% if @package_prefixes.empty? %>
        <em>(no data yet)</em>
      <% end %>
    </ul>

  </div> <!-- class='col-md-6' -->
  <div class="col-md-6">
    <h3>Search Package</h3>
    <form action='javascript:redirect();' name="searchform" id="searchform" method="post" style="display: inline;">
      <input autofocus="" id="package" name="package" size="20" type="text" value="">
      <input type="submit" value="Go">
    </form>
    </div>
  <script>
    function redirect() {
      var package = document.getElementById("package").value
      var m = package.match(/((lib)?.)/)
      prefix = m[0]
      location.href = '/packages/' + prefix + '/' + package
    }
  </script>
    
  </div> <!-- class='col-md-6' -->

</div> <!-- class='row' -->
