<div class='row'>
  <div class='col-md-12'>
    <h1 class='page-header'>Status <small>/ Alerts</small></h1>
    <%== @status_nav %>

    <h4>There are currently <b><%= @alert_number %></b> packages with status alerts.</h4>

    <% if not @tmpfail.empty? %>
      <table class='table table-striped'>
        <tr>
          <th>Package</th>
          <th>Suite</th>
          <th>Architecture</th>
          <th>Status</th>
        </tr>

      <% @tmpfail.each do |job| %>
          <tr>
            <td>
              <a href="<%= package_url(job.package) %>"><%= job.package.name %></a>
            </td>
            <td>
                <%= job.suite %>
            </td>
            <td>
                <%= job.arch %>
            </td>
            <td>
              <a href='<%= history_url(job)%>'/><%== icon(:tmpfail) %> <%= job.title %></a>
            </td>
          </tr>
      <% end %>
      </table>
    <% end %>

  </div> <!-- class='col-md-12' -->
</div> <!-- class='row' -->
